# frozen_string_literal: true
$LOAD_PATH.unshift('strum')

require 'async'
require 'async/io'
require 'async/io/tcp_socket'
require 'async/container'
require 'console'
require 'dotenv/load'
require 'singleton'


# --------------------------------------------- #
# Author: [Patrick W. <"Sickday@pm.me">] ------ #
# InitDate: [2/28/20] ------------------------- #
# --------------------------------------------- #
##
# Top level namespace for the Project.
module Strum
  VERSION = '0.9.0'
  ##
  # Network Module
  module Server
    autoload :Component,                            'strum/server/component'
    autoload :Endpoint,                             'strum/server/endpoint'
    autoload :Gateway,                              'strum/server/gateway'
    autoload :Handshake,                            'strum/server/handshake'
    autoload :Interceptor,                          'strum/server/interceptor'
    autoload :Stage,                                'strum/server/handshake'
    autoload :Manager,                              'strum/server/manager'
    ##
    # Packet Module
    module Packet
      autoload :Template,                           'strum/server/packet/template'
      autoload :Header,                             'strum/server/packet/header'
      autoload :Payload,                            'strum/server/packet/payload'
      autoload :Terminator,                         'strum/server/packet/terminator'
    end
    ##
    # Session Module
    module Session
      autoload :Template,                           'strum/server/session/template'
      autoload :Generator,                          'strum/server/session/generator'
    end
  end

  ##
  # Internal Module
  module Internal
    autoload :Hook,                                 'strum/internal/hook'
    autoload :Listener,                             'strum/internal/listener'
    autoload :Container,                            'strum/internal/container'
    autoload :Component,                            'strum/internal/component'
    autoload :Generic,                              'strum/internal/generic'
    autoload :Patches,                              'strum/internal/patches'
    ##
    # Internal Events
    module Events
      autoload :Event,                              'strum/internal/event'
      autoload :GenericEvent,                       'strum/internal/event'
      autoload :EndpointEvent,                      'strum/internal/events/endpoint'
      autoload :EndpointInitializationEvent,        'strum/internal/events/endpoint'
      autoload :EndpointListenEvent,                'strum/internal/events/endpoint'
      autoload :EndpointSpawnEvent,                 'strum/internal/events/endpoint'
      autoload :ManagerEvent,                       'strum/internal/events/manager'
      autoload :SocketEvent,                        'strum/internal/events/socket'
      autoload :SocketThrottleEvent,                'strum/internal/events/socket'
      autoload :SocketListOverflow,                 'strum/internal/events/socket'
      autoload :SocketClosedEvent,                  'strum/internal/events/socket'
      autoload :SocketReceiveEvent,                 'strum/internal/events/socket'
      autoload :SocketTimeoutEvent,                 'strum/internal/events/socket'
      autoload :SessionEvent,                       'strum/internal/events/session'
      autoload :SessionClosedEvent,                 'strum/internal/events/session'
      autoload :SessionReadEvent,                   'strum/internal/events/session'
      autoload :SessionWriteEvent,                  'strum/internal/events/session'
      autoload :SessionBuildEvent,                  'strum/internal/events/session'
      autoload :SessionRegistrationEvent,           'strum/internal/events/session'
    end
  end

  ##
  # Errors Module
  module Errors
    autoload :NilAsyncBlock,                        'strum/internal/errors'
    autoload :NilCachePath,                         'strum/internal/errors'
    autoload :NilBlockInvocation,                   'strum/internal/errors'
    autoload :NilTaskScheduled,                     'strum/internal/errors'
    autoload :PacketSizeConflict,                   'strum/internal/errors'
    autoload :ClosedPacketBuild,                    'strum/internal/errors'
    autoload :ClosedPacketIO,                       'strum/internal/errors'
    autoload :InvalidSessionRegistration,           'strum/internal/errors'
  end
  ##
  # Create a logger
  LOG = Console.logger
end