module Strum::Errors

  class ClosedGenericIO < StandardError
    def initialize
      super("IO Operation called on closed Generic receiver.")
    end
  end

  class ClosedGenericBuild < StandardError
    def initialize
      super("Build method called on closed Generic receiver.")
    end
  end

  class ClosedGenericValidation < StandardError
    def initialize
      super("Validate called on closed Generic receiver.")
    end
  end

  class PacketSizeConflict < StandardError
    def initialize(packet)
      super("Packet of Opcode: #{packet[:Data][:Header][:OperationCode]} has a header size of #{packet[:Data][:Header][:PacketSize]} while the actual size of the data is #{packet[:Data][:Payload][:Buffer].size} ")
    end
  end

  class NilAPICall < StandardError
    def initialize
      super("Event was spawned without a binding API.")
    end
  end

  class NilAsyncBlock < StandardError
    def initialize
      super('No block was provided to async execution call.')
    end
  end

  class NilBlockInvocation < StandardError
    def initialize
      super('A block was not passed to this invocation!')
    end
  end

  ##
  # Raised when a method is called by a nil Component receiver.
  class NilComponent < StandardError
    def initialize
      super('Nil component receiver')
    end
  end
  class NilComponentBlock < StandardError
    def initialize(component)
      super("Nil block invocation on component object #{component}")
    end
  end
  ##
  # Raised when a Component that relies on an Endpoint is called
  class NilEndpointComponent < StandardError
    def initialize
      super("This component cannot operate without being hooked into a valid Strum::Server::Endpoint object.")
    end
  end
  ##
  # Raised when an nil Endpoint is provided where a valid one is required.
  class NilEndpointReceiver < StandardError
    def initialize
      super("The Endpoint supplied is either nil or does not exist!")
    end
  end
  ##
  # Raised when an API Listener registers a nil event.
  class NilEventRegistration < StandardError
    def initialize
      super("Nil event provided to receiver during registration.")
    end
  end
  ##
  # Raised when Intercept is turned on, but no Interceptor is ever registered.
  class NilInterceptor < StandardError
    def initialize
      super("Intercept mode enabled, but no Interceptor has been registered to the Manager.")
    end
  end
  ##
  # Raised when a Session receiver object is nil.
  class NilSessionReceiver < StandardError
    def initialize
      super("Session event called on nil session receiver")
    end
  end
  ###
  # Raised when a Strum::Server::Handshake::Stage#execute is called with no sibling stage set.
  class NilSibling < StandardError
    def initialize(stage)
      super("The stage #{stage} has no sibling stage set.")
    end
  end
  ##
  # Raised when a Handshake is executed on a nil socket.
  class NilSocketReceiver < StandardError
    def initialize
      super("Ths Socket receiver for this Handshake is nil!")
    end
  end
  # Raised when a block passed to the Endpoint#listen method is nil.
  class NilTaskScheduled < StandardError
    def initialize
      super("Nil task scheduled to Endpoint's reactor")
    end
  end
  # Raised when the template_part param passed to Strum::Server::Packet::Template#write is not one of the following:
  # :Header, :Body, :Terminator
  class InvalidTemplatePart < StandardError
    def initialize(template_part)
      super("The following template part is invalid: #{template_part}\nPlease use any of the following instead:\n[:Header, :Body, :Terminator]")
    end
  end
  ##
  # Raised when any type that is NOT a `Strum::Server::Session` is passed to a SessionManager object.
  class InvalidSessionRegistration < StandardError
    def initialize(session)
      super("A #{session.class.name} object was passed to receiving SessionManager.")
    end
  end
  ##
  # Raised when the Strum::Server::Packet::Template#write method is called with a nil execution block.
  class NilWriteBlock < StandardError
    def initialize
      super("Write method called with no block supplied!")
    end
  end
end