# frozen_string_literal: true

# --------------------------------------------- #
# Author: [Patrick W. <"Sickday@pm.me">] ------ #
# InitDate: [3/3/20] ------------------------- #
# --------------------------------------------- #

module Strum::Internal::Events
  class Event
    include Strum::Internal::Container
    ##
    # Creates a new Event object with specified parameters and execution block
    def initialize(params = { once: false }, block)
      self[:Endpoint] =     Strum::Server::Endpoint.instance
      self[:Interceptor] =  Strum::Server::Interceptor.instance
      self[:Generator] =    Strum::Server::Session::Generator.instance
      self[:Manager] =      Strum::Server::Manager.instance
      self[:API] =          Strum::Internal::Listener.instance
      self[:SingleExec] =   params[:once]
      self[:Block] =        block
    end

    ##
    # Executes this event's block.
    def call
      raise Strum::Errors::NilBlockInvocation unless self[:Block]

      self[:Block].call(self)
    end
  end

  ##
  # Events raised by Generic Objects.
  class GenericEvent < Event
    ##
    # Executes the generic event's block.
    def call(generic)
      raise Strum::Errors::NilBlockInvocation unless self[:Block]

      self[:Generic] = generic
      self[:Block].call(self, generic)
    end
  end
end