# frozen_string_literal: true

# --------------------------------------------- #
# Author: [Patrick W. <"Sickday@pm.me">] ------ #
# InitDate: [3/3/20] -------------------------- #
# --------------------------------------------- #
# Strum
module Strum::Internal
  module Hook
    LISTENER = Strum::Internal::Listener.instance
    ##
    # Called when an Endpoint starts listening.
    def on_endpoint_listen(params = {}, &block)
      LISTENER.register_event(Strum::Internal::Events::EndpointListenEvent.new(params, block))
    end
    ##
    # Called when an Endpoint is initialized
    def on_endpoint_init(params = {}, &block)
      LISTENER.register_event(Strum::Internal::Events::EndpointInitializationEvent.new(params, block))
    end
    ##
    # Called when an Endpoint socket is spawned.
    def on_endpoint_spawn(params = {}, &block)
      LISTENER.register_event(Strum::Internal::Events::EndpointSpawnEvent.new(params, block))
    end
    ##
    # Called when a Socket triggers a delay violation.
    def on_socket_delay_violation(params = {}, &block)
      LISTENER.register_generic_event(Strum::Internal::Events::SocketDelayViolationEvent.new(params, block))
    end
    ##
    # Called when a Socket is forcefully closed.
    def on_socket_force_close(params = {}, &block)
      LISTENER.register_generic_event(Strum::Internal::Events::SocketForceCloseEvent.new(params, block))
    end
    ##
    # Called when a Socket is naturally closed.
    def on_socket_close(params = {}, &block)
      LISTENER.register_generic_event(Strum::Internal::Events::SocketClosedEvent.new(params, block))
    end
    ##
    # Called when a Socket is unable to be received because the socket list has reached capacity
    def on_socket_list_overflow(params = {}, &block)
      LISTENER.register_generic_event(Strum::Internal::Events::SocketListOverflow.new(params, block))
    end
    ##
    # Called when a Socket is throttled.
    def on_socket_throttle(params = {}, &block)
      LISTENER.register_generic_event(Strum::Internal::Events::SocketThrottleEvent.new(params, block))
    end
    ##
    # Called when a Session is registered.
    def on_session_register(params = {}, &block)
      LISTENER.register_generic_event(Strum::Internal::Events::SessionRegistrationEvent.new(params, block))
    end
    ##
    # Called when a session is built.
    def on_session_build(params = {}, &block)
      LISTENER.register_generic_event(Strum::Internal::Events::SessionBuildEvent.new(params, block))
    end
    def on_session_event(params = {}, &block)
      LISTENER.register_generic_event(Strum::Internal::Events::SessionEvent.new(params, block))
    end
    ##
    # Called when data is read from a Session's stream.
    def on_session_read(params = {}, &block)
      LISTENER.register_generic_event(Strum::Internal::Events::SessionReadEvent.new(params, block))
    end
    ##
    # Called when data is written to a Session's stream.
    def on_session_write(params = {}, &block)
      LISTENER.register_generic_event(Strum::Internal::Events::SessionWriteEvent.new(params, block))
    end
    ##
    # Called when a Session's socket is closed.
    def on_session_close(params = {}, &block)
      LISTENER.register_generic_event(Strum::Internal::Events::SessionClosedEvent.new(params, block))
    end
    ##
    # Called when a Socket's intercepted by an Interceptor.
    def on_socket_intercept(params = {}, &block)
      LISTENER.register_event(Strum::Internal::Events::InterceptedSessionEvent.new(params, block))
    end
  end
end

