# frozen_string_literal: true

# --------------------------------------------- #
# Author: [Patrick W. <"Sickday@pm.me">] ------ #
# InitDate: [3/4/20] ------------------------- #
# --------------------------------------------- #

module Strum::Internal
  class Listener
    include Strum::Internal::Component
    include Singleton

    ##
    # Creates a new API Hook
    def initialize
      super(:Container)
      self[:Events] = {}
      load_events
    end

    ##
    # Fires events of a specific type
    def trigger_events(klass)
      report "#{klass.name.split('::').last.to_sym} event raised!" unless ENV['EVENT_DEBUG'].to_i.zero?
      unless klass.nil?
        self[:Events][klass.name.split('::').last.to_sym].each do |event|
          unless event.nil?
            event.call
            self[:Events][klass.name.split('::').last.to_sym].delete(event) if event[:SingleExec]
          end
        end
      end
    end

    ##
    # Fires events of a specific type
    def trigger_generic_events(generic, klass)
      report "#{klass.name.split('::').last.to_sym} Generic Event raised!" unless ENV['EVENT_DEBUG'].to_i.zero?
      unless klass.nil?
        self[:GenericEvents][klass.name.split('::').last.to_sym].each do |event|
          unless event.nil?
            event.call(generic)
            self[:GenericEvents][klass.name.split('::').last.to_sym].delete(event) if event[:SingleExec]
          end
        end
      end
    end

    ##
    # Registers an Event to this Listener.
    def register_event(event)
      raise Strum::Errors::NilEventRegistration unless event

      self[:Events][event.class.name.split('::').last.to_sym] << event
    end

    ##
    # Registers an Event to this Listener.
    def register_generic_event(event)
      raise Strum::Errors::NilEventRegistration unless event

      self[:GenericEvents][event.class.name.split('::').last.to_sym] << event
    end

    private
    ##
    # Loads our predefined events.
    def load_events
      Dir["events/*.rb"].sort.each { |event| require event }
      self[:Events] = {
          EndpointAcceptEvent: [],
          EndpointInitializationEvent: [],
          EndpointListenEvent: [],
          EndpointSocketRegistrationEvent: [],
          EndpointSpawnEvent: [],
          EndpointEvent: [],
          ManagerEvent: [],
      }
      self[:GenericEvents] = {
          SessionEvent: [],
          SessionClosedEvent: [],
          SessionReadEvent: [],
          SessionRegistrationEvent: [],
          SessionBuildEvent: [],
          SocketEvent: [],
          SocketThrottleEvent: [],
          SocketListOverflow: [],
          SocketClosedEvent: [],
          SocketReceiveEvent: [],
          SocketTimeoutEvent: []
      }
    end
  end
end
