# frozen_string_literal: true

#####
# Author: Patrick "Sickday" W.
# Date: 1/3/2020
# Rev: 2/13/2020
######
# # refinements that make changes to Ruby Core objects.
module Strum::Internal::Patches
  # Adds support for IO primitives
  # Don't pollute the global namespace :thumbs_up:
  refine Integer do

    ##
    # Returned unsigned
    def unsigned(type)
      0 unless self.positive?
      case type
      when :Byte    || :byte  || :b   || :B
        adjust(self.to_i & 0xff, 8)
      when :Short   || :short || :s   || :S
        adjust(self.to_i & 0xffff, 16)
      when :Integer || :Int   || :int || :integer || :i || :I
        adjust(self.to_i & 0xffffffff)
      when :Long    || :long  || :l   || :L
        adjust(self.to_i & 0xffffffffffffffff, 64)
      else
        adjust(self.to_i &  0xffffffff)
      end
    end

    ## Shorthand
    alias :u :unsigned

    ##
    # Return signed
    def signed(type)
      case type
      when :Byte    || :byte  || :b   || :B
        adjust(self.to_i, 7)
      when :Short   || :short || :s   || :S
        adjust(self.to_i, 15)
      when :Integer || :Int   || :int || :integer || :i || :I
        adjust(self.to_i)
      when :Long    || :long  || :l   || :L
        adjust(self.to_i, 64)
      else
        adjust(self)
      end
    end

    ## Shorthand
    alias :s :signed

    def nibble
      adjust(self.to_i & 0xf, 4)
    end
    ##
    # adjusts this Integer
    private def adjust(num, exponent = 31)
      integer = num
      bits = (Math.log(2**exponent) / Math.log(2)).to_i + 1
      over_flow = (2**bits) - 1

      integer < -2**exponent ? integer & over_flow : integer
      integer > 2**exponent - 1 ? -1 * (-integer & over_flow) : integer
    end
  end
end