# frozen_string_literal: true

# --------------------------------------------- #
# Author: [Patrick W. <"Sickday@pm.me">] ------ #
# InitDate: [3/3/20] ------------------------- #
# --------------------------------------------- #

module Strum::Server
  class Handshake
    include Strum::Internal::Container
    ##
    # Create a new Handshake object
    # @param socket [Async::IO::TCPSocket] the socket this handshake will cycling stages through.
    # @param stages [Array] All stages for this Handshake.
    def initialize(socket, *stages)
      raise Strum::Errors::NilSocketReceiver unless socket

      self[:Socket] = socket
      self[:StageList] = {}
      stages.each do |stage|
        self[:StageList][stages.index(stage)] = stage.new(self[:Socket], self)
        self[:StageList][stages.index(stage)]
            .register_sibling(stages[stages.index(stage) + 1])
      end
    end

    ##
    # Cycles through the Handshake stages.
    #
    def cycle
      set_stage(self[:StageList].first)
    end

    ##
    # Sets the current stage.
    # @param stage [Stage] the stage we're setting this Handshake to.
    def set_stage(stage)
      raise Strum::Errors::NilSocketReceiver unless self[:Socket]

      self[:Stage] = stage.new(self[:Socket], self)
      self[:Stage].execute
    end

  end

  ##
  # A Handshake stage.
  class Stage
    ##
    # Create a new Handshake stage with the supplied socket.
    # @param socket [Async::IO::TCPSocket] - This Stage's socket
    # @param handshake [Handshake] - The Handshake for this stage.
    def initialize(socket, handshake)
      self[:Socket] = socket
      self[:Handshake] = handshake
    end

    ##
    # execute the stage
    def execute
      raise Strum::Errors::NilSibling unless self[:Next]

      self[:Handshake].set_stage(self[:Next])
    end

    ##
    # Make this Stage aware of the next stage in the cycle.
    # @param sibling [Stage] The sibling of this stage.
    def register_sibling(sibling)
      self[:Next] = sibling
    end
  end
end
