# frozen_string_literal: true

# --------------------------------------------- #
# Author: [Patrick W. <"Sickday@pm.me">] ------ #
# InitDate: [3/5/20] ------------------------- #
# --------------------------------------------- #

module Strum::Server
  class Manager
    include Strum::Internal::Component
    include Singleton
    ##
    # Creates a new SessionManager.
    def initialize(data = {})
      self[:Components] = {Generator: Strum::Server::Session::Generator.instance,
                           Endpoint: Strum::Server::Endpoint.instance,
                           API: Strum::Internal::Listener.instance }
      self[:SessionList] = {}
      hook_components
      self[:Components][:API].trigger_events(Strum::Internal::Events::ManagerEvent)
      super(:Container)
    end

    ##
    # Registers a Session to this SessionManager
    # @param session [Strum::Server::Session::Template] - the session we're registering.
    def register_session(session)
      self[:Components][:API].trigger_events(Strum::Internal::Events::ManagerEvent)
      raise Strum::Errors::InvalidSessionRegistration unless session.is_a? Strum::Server::Session::Template
      if self[:SessionList].keys.include?(session[:IP])
        self[:SessionList][session[:IP]] << session
      else
        self[:SessionList][session[:IP]] = [session]
      end
      self[:Components][:API].trigger_generic_events(session, Strum::Internal::Events::SessionRegistrationEvent)
    end

    ##
    # Get the current connection Count
    def connection_count
      size = 0
      self[:SessionList].values.each { |socket_list| size =+ socket_list.size}
      size
    end

    ##
    # Registers an interceptor to this manager.
    def register_interceptor(klass)
      self[:Components][:Interceptor] = klass.instance
    end

    ##
    # Test against all sockets.
    def any_sockets?(&block)
      raise Strum::Errors::NilBlockInvocation unless block_given?

      self[:SessionList].any? do |_ip, sockets|
        sockets.any? { |socket| block.call(socket)}
      end
    end

    ##
    # Receives a socket
    # @param socket [TCPSocket] the Socket object we're receiving
    def receive(socket)
      session = self[:Components][:Generator].generate(socket)
      ## Intercept the socket and pass to our Interceptor

      unless ENV['INTERCEPT_MODE'].to_i.zero?
        raise Strum::Errors::NilInterceptor unless self[:Interceptor]
        self[:Components][:Interceptor].intercept(session)
      end

      register_session(session)

      self[:Components][:API].trigger_events(Strum::Internal::Events::ManagerEvent)
    end

    private

    ##
    # Hook all components
    def hook_components
      self[:Components].each {|_name, component| component.hook(:Manager, self)}
    end
  end
end