# frozen_string_literal: true

# --------------------------------------------- #
# Author: [Patrick W. <"Sickday@pm.me">] ------ #
# InitDate: [3/3/20] ------------------------- #
# --------------------------------------------- #

module Strum::Server::Packet
  ##
  # A Packet Header object.
  class Header
    using Strum::Internal::Patches
    include Strum::Internal::Generic
    ##
    # Create a new Header.
    def initialize(data = {})
      self[:OperationCode] = data[:op_code]
      self[:PacketSize]    = data[:size]
      self[:Buffer] = Async::IO::Buffer.new
    end

    ##
    # Build's this header.
    # @param type [Symbol] - How this Header should be packed. (:Byte, :Short)
    def build(type)
      super
      case type
      when :Byte
        self[:Buffer] << [self[:OperationCode].unsigned(:Byte),
                          self[:PacketSize].unsigned(:Short)].pack('CC')
      when :Short
        self[:Buffer] << [self[:OperationCode].unsigned(:Byte),
                          self[:PacketSize].unsigned(:Short)].pack('nn')
      else
        self[:Buffer] << [self[:OperationCode].unsigned(:Byte),
                          self[:PacketSize].unsigned(:Short)].pack('CC')
      end
      self[:Buffer]
    end

    def write(data = {})
      self[:OperationCode] = data[:OperationCode] if data[:OperationCode]
      self[:PacketSize] = data[:PacketSize] if data[:PacketSize]
    end

    alias_method :<<, :write
  end
end