# frozen_string_literal: true

# --------------------------------------------- #
# Author: [Patrick W. <"Sickday@pm.me">] ------ #
# InitDate: [3/3/20] ------------------------- #
# --------------------------------------------- #

module Strum::Server::Session
  class Template
    include Strum::Internal::Generic
    ##
    # Creates a new Session Template object
    def initialize(data = {})
      self[:Socket]     = data[:Socket]
      self[:IP]         = data[:Socket].io.addr[-1]
      self[:InitTime]   = data[:InitTime]
      self[:BuildTime]  = data[:BuildTime]
      self[:Status]     = :ACTIVE
      self[:API] = Strum::Internal::Listener.instance
      self[:API].trigger_generic_events(self, Strum::Internal::Events::SessionBuildEvent)
    end
    ##
    # Writes data to the underlying stream.
    def write(data, &block)
      self[:API].trigger_events(Strum::Internal::Events::SessionWriteEvent)
      unless closed?
        if data
          begin
            self[:Socket].write(data)
          rescue StreamError
            report "An error was encountered while writing to the stream of socket #{self[:IP]}"
          end
        end
      end
      block.call(self[:Socket]) if block_given?
    end
    ##
    # Reads data from the underlying stream.
    def read(length)
      self[:API].trigger_events(Strum::Internal::Events::SessionReadEvent)
      unless closed?
        begin
          self[:Socket].read(length)
        rescue StreamError
          report "An error was encountered while reading from the stream of socket #{self[:IP]}"
        end
      end
    end
    ##
    # Closes this socket and marks this Generic as closed.
    def close!
      self[:API].trigger_events(Strum::Internal::Events::SessionClosedEvent)
      self[:Socket].close
      super
    end
    ##
    # Checks if this Session is (or should be) closed.
    def closed?
      true if self[:Status] == :CLOSED
      if self[:Socket].closed?
        close!
        true
      end
    end
    ##
    # Nothing to see here <.<
    private def close; end
  end
end